@@ -1,4 +1,3 @@
-import os
 from gql import gql
 
 from gql import Client
@@ -50,8 +49,6 @@ class SubgraphClient:
               maxOI
               makerFeeP
               takerFeeP
-              usageFeeP
-              utilizationThresholdP
               makerMaxLeverage    
               curFundingLong  
               curFundingShort
@@ -65,18 +62,15 @@ class SubgraphClient:
               accFundingShort
               lastFundingBlock
               maxFundingFeePerBlock
-              lastFundingRate
-              maxOI
+              lastFundingRate              
               hillInflectionPoint
               hillPosScale
               hillNegScale
-              lastOiDelta
               springFactor
               sFactorUpScaleP
               sFactorDownScaleP
               lastTradePrice
-              maxLeverage
-              lastFundingVelocity
+              maxLeverage              
               group {
                 id
                 name
@@ -105,8 +99,23 @@ class SubgraphClient:
         else:
             raise ValueError(f"No pair details found for pair ID: {pair_id}")
 
+    async def get_liq_margin_threshold_p(self):
+        self.log(f"Fetching get_liq_margin_threshold_p")
+        query = gql(
+            """
+          query metaDatas {
+            metaDatas {              
+              liqMarginThresholdP
+            }
+          }
+          """
+        )
+        result = await self.client.execute_async(query)
+
+        return result['metaDatas'][0]['liqMarginThresholdP']
+
     async def get_open_trades(self, address):
-        self.log(f"Fetching open trades for address: {address}")
+        # self.log(f"Fetching open trades for address: {address}")
         query = gql(
             """
           query trades($trader: Bytes!) {
@@ -116,6 +125,7 @@ class SubgraphClient:
           tradeID
           collateral
           leverage
+          highestLeverage
           openPrice
           stopLossPrice
           takeProfitPrice
@@ -138,14 +148,21 @@ class SubgraphClient:
             rolloverFeePerBlock
             accFundingLong
             spreadP
-            tradeSizeRef
             accFundingShort
             longOI
             shortOI
+            maxOI
+            maxLeverage
+            hillInflectionPoint
+            hillPosScale
+            hillNegScale
+            springFactor
+            sFactorUpScaleP
+            sFactorDownScaleP
             lastFundingBlock
             maxFundingFeePerBlock
-            lastFundingVelocity
             lastFundingRate
+            maxLeverage
           }
         }
       }
@@ -184,13 +201,11 @@ class SubgraphClient:
                 rolloverFeePerBlock
                 accFundingLong
                 spreadP
-                tradeSizeRef
                 accFundingShort
                 longOI
                 shortOI
                 lastFundingBlock
                 maxFundingFeePerBlock
-                lastFundingVelocity
                 lastFundingRate
               }
             }
@@ -250,3 +265,106 @@ class SubgraphClient:
         result = await self.client.execute_async(query, variable_values={"trader": trader, "last_n_orders": last_n_orders})
 
         return list(reversed(result['orders']))  # Reverse the final list
+
+    async def get_order_by_id(self, order_id):
+        """
+        Get an order by its ID
+        """
+        query = gql(
+            """
+            query GetOrder($order_id: ID!) {
+              orders(where: {id: $order_id}) {
+                id
+                trader
+                pair {
+                  id
+                  from
+                  to
+                  feed
+                }
+                tradeID
+                limitID
+                orderType
+                orderAction
+                price
+                priceAfterImpact
+                priceImpactP
+                collateral
+                notional
+                tradeNotional
+                profitPercent
+                totalProfitPercent
+                amountSentToTrader
+                isBuy
+                initiatedAt
+                executedAt
+                initiatedTx
+                executedTx
+                initiatedBlock
+                executedBlock
+                leverage
+                isPending
+                isCancelled
+                cancelReason
+                devFee
+                vaultFee
+                oracleFee
+                liquidationFee
+                fundingFee
+                rolloverFee
+                closePercent
+              }
+            }
+            """
+        )
+
+        result = await self.client.execute_async(query, variable_values={"order_id": str(order_id)})
+
+        if result and 'orders' in result and len(result['orders']) > 0:
+            return result['orders'][0]
+        return None
+
+    async def get_trade_by_id(self, trade_id):
+        """
+        Get a trade by its ID
+        """
+        query = gql(
+            """
+            query GetTrade($trade_id: ID!) {
+              trades(where: {id: $trade_id}) {
+                id
+                trader
+                pair {
+                  id
+                  from
+                  to
+                  feed
+                }
+                index
+                tradeID
+                tradeType
+                openPrice
+                closePrice
+                takeProfitPrice
+                stopLossPrice
+                collateral
+                notional
+                tradeNotional
+                highestLeverage
+                leverage
+                isBuy
+                isOpen
+                closeInitiated
+                funding
+                rollover
+                timestamp
+              }
+            }
+            """
+        )
+
+        result = await self.client.execute_async(query, variable_values={"trade_id": str(trade_id)})
+
+        if result and 'trades' in result and len(result['trades']) > 0:
+            return result['trades'][0]
+        return None
